Method: Fiddle::Function#call
- Defined in:
- ext/fiddle/function.c
#call(argv[], self) ⇒ Object
Calls the constructed Function, with args. Caller must ensure the underlying function is called in a thread-safe manner if running in a multi-threaded process.
Note that it is not thread-safe to use this method to directly or indirectly call many Ruby C-extension APIs unless you don’t pass need_gvl: true to Fiddle::Function#new.
For an example see Fiddle::Function
209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 |
# File 'ext/fiddle/function.c', line 209
static VALUE
function_call(int argc, VALUE argv[], VALUE self)
{
struct nogvl_ffi_call_args args = { 0 };
fiddle_generic *generic_args;
VALUE cfunc;
VALUE abi;
VALUE arg_types;
VALUE cPointer;
VALUE is_variadic;
VALUE need_gvl;
int n_arg_types;
int n_fixed_args = 0;
int n_call_args = 0;
int i;
int i_call;
VALUE converted_args = Qnil;
VALUE alloc_buffer = 0;
cfunc = rb_iv_get(self, "@ptr");
abi = rb_iv_get(self, "@abi");
arg_types = rb_iv_get(self, "@argument_types");
cPointer = rb_const_get(mFiddle, rb_intern("Pointer"));
is_variadic = rb_iv_get(self, "@is_variadic");
need_gvl = rb_iv_get(self, "@need_gvl");
n_arg_types = RARRAY_LENINT(arg_types);
n_fixed_args = n_arg_types;
if (RTEST(is_variadic)) {
if (argc < n_arg_types) {
rb_error_arity(argc, n_arg_types, UNLIMITED_ARGUMENTS);
}
if (((argc - n_arg_types) % 2) != 0) {
rb_raise(rb_eArgError,
"variadic arguments must be type and value pairs: "
"%"PRIsVALUE,
rb_ary_new_from_values(argc, argv));
}
n_call_args = n_arg_types + ((argc - n_arg_types) / 2);
}
else {
if (argc != n_arg_types) {
rb_error_arity(argc, n_arg_types, n_arg_types);
}
n_call_args = n_arg_types;
}
Check_Max_Args("the number of arguments", n_call_args);
TypedData_Get_Struct(self, ffi_cif, &function_data_type, args.cif);
if (is_variadic && args.cif->arg_types) {
xfree(args.cif->arg_types);
args.cif->arg_types = NULL;
}
if (!args.cif->arg_types) {
VALUE fixed_arg_types = arg_types;
VALUE return_type;
int c_return_type;
ffi_type *ffi_return_type;
ffi_type **ffi_arg_types;
ffi_status result;
arg_types = rb_ary_dup(fixed_arg_types);
for (i = n_fixed_args; i < argc; i += 2) {
VALUE arg_type = argv[i];
int c_arg_type;
arg_type = rb_fiddle_type_ensure(arg_type);
c_arg_type = NUM2INT(arg_type);
(void)INT2FFI_TYPE(c_arg_type); /* raise */
rb_ary_push(arg_types, INT2FIX(c_arg_type));
}
return_type = rb_iv_get(self, "@return_type");
c_return_type = FIX2INT(return_type);
ffi_return_type = INT2FFI_TYPE(c_return_type);
ffi_arg_types = xcalloc(n_call_args + 1, sizeof(ffi_type *));
for (i_call = 0; i_call < n_call_args; i_call++) {
VALUE arg_type;
int c_arg_type;
arg_type = RARRAY_AREF(arg_types, i_call);
c_arg_type = FIX2INT(arg_type);
ffi_arg_types[i_call] = INT2FFI_TYPE(c_arg_type);
}
ffi_arg_types[i_call] = NULL;
if (is_variadic) {
#ifdef HAVE_FFI_PREP_CIF_VAR
result = ffi_prep_cif_var(args.cif,
FIX2INT(abi),
n_fixed_args,
n_call_args,
ffi_return_type,
ffi_arg_types);
#else
/* This code is never used because ffi_prep_cif_var()
* availability check is done in #initialize. */
result = FFI_BAD_TYPEDEF;
#endif
}
else {
result = ffi_prep_cif(args.cif,
FIX2INT(abi),
n_call_args,
ffi_return_type,
ffi_arg_types);
}
if (result != FFI_OK) {
xfree(ffi_arg_types);
args.cif->arg_types = NULL;
rb_raise(rb_eRuntimeError, "error creating CIF %d", result);
}
}
generic_args = ALLOCV(alloc_buffer,
sizeof(fiddle_generic) * n_call_args +
sizeof(void *) * (n_call_args + 1));
args.values = (void **)((char *)generic_args +
sizeof(fiddle_generic) * n_call_args);
for (i = 0, i_call = 0;
i < argc && i_call < n_call_args;
i++, i_call++) {
VALUE arg_type;
int c_arg_type;
VALUE original_src;
VALUE src;
arg_type = RARRAY_AREF(arg_types, i_call);
c_arg_type = FIX2INT(arg_type);
if (i >= n_fixed_args) {
i++;
}
src = argv[i];
if (c_arg_type == TYPE_VOIDP) {
if (NIL_P(src)) {
src = INT2FIX(0);
}
else if (cPointer != CLASS_OF(src)) {
src = rb_funcall(cPointer, rb_intern("[]"), 1, src);
if (NIL_P(converted_args)) {
converted_args = rb_ary_new();
}
rb_ary_push(converted_args, src);
}
src = rb_Integer(src);
}
original_src = src;
VALUE2GENERIC(c_arg_type, src, &generic_args[i_call]);
if (src != original_src) {
if (NIL_P(converted_args)) {
converted_args = rb_ary_new();
}
rb_ary_push(converted_args, src);
}
args.values[i_call] = (void *)&generic_args[i_call];
}
args.values[i_call] = NULL;
args.fn = (void(*)(void))NUM2PTR(cfunc);
if (RTEST(need_gvl)) {
ffi_call(args.cif, args.fn, &(args.retval), args.values);
}
else {
(void)rb_thread_call_without_gvl(nogvl_ffi_call, &args, 0, 0);
}
{
int errno_keep = errno;
#if defined(_WIN32)
int socket_error = WSAGetLastError();
rb_funcall(mFiddle, rb_intern("win32_last_error="), 1,
INT2NUM(errno_keep));
rb_funcall(mFiddle, rb_intern("win32_last_socket_error="), 1,
INT2NUM(socket_error));
#endif
rb_funcall(mFiddle, rb_intern("last_error="), 1, INT2NUM(errno_keep));
}
ALLOCV_END(alloc_buffer);
return GENERIC2VALUE(rb_iv_get(self, "@return_type"), args.retval);
}
|